@@ -37,13 +37,7 @@ class CatchHandler:
     result_path: str = "$.error-info"
 
 
-def concurrent(
-    iterable,
-    max_concurrency: Optional[int] = None,
-    distributed=False,
-    tolerated_failure_percentage=None,
-    tolerated_failure_count=None,
-):
+def concurrent(iterable, max_concurrency: Optional[int] = None):
     pass
 
 
@@ -282,10 +276,133 @@ class SFNScope:
         for handler in stmt.handlers:
             child_scope = ChildScope(self)
             result_path = None
+            h_chain, h_n = child_scope.handle_body(handler.body)
             if handler.name:
                 result_path = f"$.register.{handler.name}"
                 child_scope.add_var(handler.name)
-            h_chain, h_n = child_scope.handle_body(handler.body)
+                if h_chain:
+                    process_exception = sfn.Pass(
+                        self.cdk_stack,
+                        self.state_name(f"Interpret {handler.name}"),
+                        input_path="$.register",
+                        parameters=child_scope.build_register_assignment(
+                            {
+                                handler.name: JsonPath.string_at(
+                                    f"States.StringToJson($.{handler.name}.Cause)"
+                                )
+                            }
+                        ),
+                        result_path="$.register",
+                    )
+                    process_exception_choice = sfn.Choice(
+                        self.cdk_stack,
+                        self.state_name(f"Interpret Choice {handler.name}"),
+                    )
+                    process_exception_cause = sfn.Pass(
+                        self.cdk_stack,
+                        self.state_name(f"Interpret {handler.name}"),
+                        input_path="$.register",
+                        parameters=child_scope.build_register_assignment(
+                            {
+                                handler.name: {
+                                    "Cause": JsonPath.string_at(
+                                        f"States.StringToJson($.{handler.name}.Cause)"
+                                    ),
+                                    "Error": JsonPath.string_at(
+                                        f"$.{handler.name}.Error"
+                                    ),
+                                    "ExecutionArn": JsonPath.string_at(
+                                        f"$.{handler.name}.ExecutionArn"
+                                    ),
+                                    "Name": JsonPath.string_at(
+                                        f"$.{handler.name}.Name"
+                                    ),
+                                    "Status": JsonPath.string_at(
+                                        f"$.{handler.name}.Status"
+                                    ),
+                                }
+                            }
+                        ),
+                        result_path="$.register",
+                    )
+                    format_string = "{}{}"
+                    split = "{"
+                    process_exception_malformed_cause = sfn.Pass(
+                        self.cdk_stack,
+                        self.state_name(f"Interpret {handler.name}"),
+                        input_path="$.register",
+                        parameters=child_scope.build_register_assignment(
+                            {
+                                handler.name: {
+                                    "Cause": JsonPath.string_at(
+                                        f"States.StringToJson(States.Format('{format_string}', '{split}', States.ArrayGetItem(States.StringSplit($.{handler.name}.Cause, '{split}'), 1)))"
+                                    ),
+                                    "Error": JsonPath.string_at(
+                                        f"$.{handler.name}.Error"
+                                    ),
+                                    "ExecutionArn": JsonPath.string_at(
+                                        f"$.{handler.name}.ExecutionArn"
+                                    ),
+                                    "Name": JsonPath.string_at(
+                                        f"$.{handler.name}.Name"
+                                    ),
+                                    "Status": JsonPath.string_at(
+                                        f"$.{handler.name}.Status"
+                                    ),
+                                }
+                            }
+                        ),
+                        result_path="$.register",
+                    )
+                    process_exception_default = sfn.Pass(
+                        self.cdk_stack,
+                        self.state_name(f"Interpret {handler.name}"),
+                        input_path="$.register",
+                        parameters=child_scope.build_register_assignment(
+                            {
+                                handler.name: {
+                                    "ExecutionArn": JsonPath.string_at(
+                                        f"$.{handler.name}.ExecutionArn"
+                                    ),
+                                    "Name": JsonPath.string_at(
+                                        f"$.{handler.name}.Name"
+                                    ),
+                                    "Status": JsonPath.string_at(
+                                        f"$.{handler.name}.Status"
+                                    ),
+                                }
+                            }
+                        ),
+                        result_path="$.register",
+                    )
+                    process_exception.next(process_exception_choice)
+                    process_exception_choice.when(
+                        sfn.Condition.and_(
+                            sfn.Condition.is_present(
+                                f"$.register.{handler.name}.Cause"
+                            ),
+                            sfn.Condition.string_matches(
+                                f"$.register.{handler.name}.Cause",
+                                "The cause could not be determined*",
+                            ),
+                        ),
+                        process_exception_malformed_cause,
+                    )
+                    process_exception_choice.when(
+                        sfn.Condition.is_present(f"$.register.{handler.name}.Cause"),
+                        process_exception_cause,
+                    )
+                    process_exception_choice.otherwise(process_exception_default)
+                    process_exception_cause.next(h_chain[0])
+                    process_exception_malformed_cause.next(h_chain[0])
+                    process_exception_default.next(h_chain[0])
+                    h_chain = [
+                        process_exception,
+                        process_exception_choice,
+                        process_exception_cause,
+                        process_exception_malformed_cause,
+                        process_exception_default,
+                    ] + h_chain
 
             # If the handler body isn't empty, add it in
             if h_chain:
@@ -355,9 +472,6 @@ class SFNScope:
         # TODO: Support callable iterator
         max_concurrency = 1
         iterator_step = None
-        distributed = False
-        tol_fail_perc = None
-        tol_fail_count = None
 
         # if the iterator is wrapped in a 'concurrent' function, capture the parameter as the concurrency for the map
         if (
@@ -365,19 +479,10 @@ class SFNScope:
             and isinstance(iterator.func, ast.Name)
             and iterator.func.id == "concurrent"
         ):
-            args, kwargs = get_call_args(iterator)
-            m_c = kwargs.get("max_concurrency")
-            if len(args) > 1 and isinstance(args[1], ast.Constant):
-                # TODO: Check for type other than constant
-                max_concurrency = args[1].value
-            if m_c and isinstance(m_c, ast.Constant):
-                max_concurrency = m_c.value
+            if len(iterator.args) > 1:
+                max_concurrency = iterator.args[1].value
             else:
                 max_concurrency = 0
-            distributed = kwargs.get("distributed", False)
-            tol_fail_perc = kwargs.get("tolerated_failure_percentage")
-            tol_fail_count = kwargs.get("tolerated_failure_count")
-
             iterator = iterator.args[0]
 
         if isinstance(iterator, ast.Call) and self._is_intrinsic_function(iterator):
@@ -407,26 +512,12 @@ class SFNScope:
             items_path = f"$.register.{iter_var}"
         else:
             raise Exception("Unsupported for-loop iterator, variables only")
-        return (
-            iter_var,
-            items_path,
-            iterator_step,
-            max_concurrency,
-            distributed,
-            tol_fail_perc,
-            tol_fail_count,
-        )
+        return iter_var, items_path, iterator_step, max_concurrency
 
     def handle_for(self, stmt: ast.For):
-        (
-            iter_var,
-            items_path,
-            iterator_step,
-            max_concurrency,
-            distributed,
-            tol_fail_perc,
-            tol_fail_count,
-        ) = self._get_iterator(stmt.iter)
+        iter_var, items_path, iterator_step, max_concurrency = self._get_iterator(
+            stmt.iter
+        )
         if not isinstance(stmt.target, ast.Name):
             raise Exception("Unsupported for-loop target, variables only")
 
@@ -1125,15 +1216,15 @@ class MapScope(SFNScope):
     def __init__(self, parent_scope: SFNScope):
         super(MapScope, self).__init__(parent_scope.fts)
         self.variables = parent_scope.variables.copy()
-        self.scoped_variables = []
-        self._updated_vars = []
+        self.scoped_variables = set()
+        self._updated_vars = set()
         self.parent_scope = parent_scope
 
     def _added_var(self, var: str):
-        self.scoped_variables.append(var)
+        self.scoped_variables.add(var)
 
     def _updated_var(self, var: str):
-        self._updated_vars.append(var)
+        self._updated_vars.add(var)
 
     def build_entry_step(self, entry_var: str):
         return sfn.Pass(
@@ -1159,6 +1250,8 @@ class ChildScope(SFNScope):
 
     def _added_var(self, var: str):
         self.scoped_variables.append(var)
+        if self.parent_scope:
+            self.parent_scope._added_var(var)
 
 
 def advance(
